package org.algorithm.core.cnn.model;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import org.algorithm.core.cnn.AlgorithmCNNExecutor;
import org.algorithm.core.cnn.dataset.RelationExtractionDataSet;
import org.tensorflow.SavedModelBundle;
import org.tensorflow.Session;
import org.tensorflow.Tensor;

import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.util.ArrayList;
import java.util.List;

/**
 * @Author: bijl
 * @Date: 2019/1/22 10:21
 * @Decription:
 */
public class RelationExtractionModel extends AlgorithmCNNExecutor {
//    self.X = tf.placeholder(tf.int32, shape=[None, self.max_length], name='X')
//    self.pos1 = tf.placeholder(tf.int32, shape=[None, self.max_length], name='pos1')
//    self.pos2 = tf.placeholder(tf.int32, shape=[None, self.max_length], name='pos2')
//
//    self.y = tf.placeholder(tf.float32, shape=[None, self.n_classes], name='y')
//    self.keep_prob = tf.placeholder(dtype=tf.float32, name='keep_prob')
    private final String X_PLACEHOLDER = "X";
    private final String pos1_PLACEHOLDER = "pos1";
    private final String pos2_PLACEHOLDER = "pos2";
    private final String SOFT_MAX = "softmax/Softmax";
    private final int NUM_LABEL = 2;
    private SavedModelBundle bundle; // 模型捆绑
    private Session session;  // 会话
    private RelationExtractionDataSet dataSet;


    /**
     *
     * @param exportDir 模型保存地址
     */
    public RelationExtractionModel(String exportDir, RelationExtractionDataSet dataSet){
        this.dataSet = dataSet;
        this.init(exportDir);
    }

    /**
     * 初始化：加载模型，获取会话。
     * @param exportDir
     */
    public void init(String exportDir) {
        /* load the model Bundle */
        try {
            this.bundle = SavedModelBundle.load(exportDir, "serve");
        } catch (Exception e) {
            e.printStackTrace();
        }

        // create the session from the Bundle
        this.session = bundle.session();
    }

    @Override
    public List<Triad> execute(String content, String json_content) {
        List<LemmaInfo[]> combinations = new ArrayList<>();
        List<LemmaInfo> lemmaInfos = this.stringToObjects(json_content);  // json数组字符串装对象list
        // 组合
        for(int i=0; i < lemmaInfos.size() - 1; i++){  // 两两组合成实体对
            for (int j = i + 1; j< lemmaInfos.size(); j++){
                LemmaInfo[] pair = new LemmaInfo[2];
                pair[0] = lemmaInfos.get(i);
                pair[1] = lemmaInfos.get(j);
                combinations.add(pair);
            }
        }

        List<Triad> triads = new ArrayList<>();

        // 遍历组合
        for (LemmaInfo[] lemmaInfoPair: combinations) {
            int[][] example = dataSet.getExample(content, lemmaInfoPair[0], lemmaInfoPair[1]);
            // 调用模型
            float[][] relation = this.run(example, 1);
            Triad triad = new Triad();

            // TODO:修改triad
            // 生成Triad（三元组）
            triad.setL_1(lemmaInfoPair[0].toLemma());
            triad.setL_2(lemmaInfoPair[1].toLemma());
            triad.setRelation(relation[0][0] > relation[0][1] ? "无":"有");
            triads.add(triad);
        }
        return triads;
    }

    /**
     * json数组字符串装对象list
     * @param json_content
     * @return
     */
    public List<LemmaInfo> stringToObjects(String json_content) {
        List<LemmaInfo> lemmaInfos = new ArrayList<>();
        JSONArray jsonArray = JSONArray.parseArray(json_content);
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject job = jsonArray.getJSONObject(i);
            LemmaInfo info = JSON.parseObject(job.toJSONString(), new TypeReference<LemmaInfo>() {
            });
            lemmaInfos.add(info);
        }
        return lemmaInfos;
    }


    /**
     *
     * @param inputValues 字符id，相对于实体1位置，相对于实体2位置
     * @param batchSize 批量大小
     * @return
     */
    private float[][] run(int[][] inputValues, int batchSize){
        long[] shape = {1, dataSet.maxLength};  // 老模型
        Tensor<Integer> charId = Tensor.create(
                shape,
                IntBuffer.wrap(inputValues[0])
        );
        Tensor<Integer> pos1 = Tensor.create(
                shape,
                IntBuffer.wrap(inputValues[1])
        );
        Tensor<Integer> pos2 = Tensor.create(
                shape,
                IntBuffer.wrap(inputValues[2])
        );

        return this.session.runner()
                .feed(this.X_PLACEHOLDER, charId)
                .feed(this.pos1_PLACEHOLDER, pos1)
                .feed(this.pos2_PLACEHOLDER, pos2)
                .feed("keep_prob", Tensor.create(1.0f, Float.class))  // dropout保留率
                .fetch(this.SOFT_MAX).run().get(0)
                .copyTo(new float[1][this.NUM_LABEL]);
    }

    /**
     * 关闭会话,释放资源
     */
    public void close() {
        this.session.close();
        this.bundle.close();
    }
}
