package org.diagbot.nlp.controller;

import org.algorithm.core.cnn.AlgorithmCNNExecutor;
import org.algorithm.core.cnn.model.Triad;
import org.apache.commons.lang3.StringUtils;
import org.diagbot.nlp.feature.FeatureAnalyze;
import org.diagbot.nlp.feature.FeatureType;

import java.util.*;

/**
 * @ClassName org.diagbot.nlp.controller.RelationExtractionController
 * @Description TODO
 * @Author fyeman
 * @Date 2019/1/17/017 11:21
 * @Version 1.0
 **/
public class RelationExtractionController {

    public void extraction(String content) throws Exception {
        List<String> features = extractEntity(content);
        AlgorithmCNNExecutor executor = new AlgorithmCNNExecutor();
        List<Triad> triads = executor.execute(content);

        for (String feature : features) {
            lookRelationFeatures(feature, triads);
        }
    }

    private void lookRelationFeatures(String feature, List<Triad> triads) {
        for (Triad triad : triads) {
            if (triad.getL_1().getText().equals(feature)) {
                lookRelationFeatures(triad.getL_2().getText(), triads);
            } else if (triad.getL_2().getText().equals(feature)) {
                lookRelationFeatures(triad.getL_1().getText(), triads);
            } else {

            }
        }
    }

    public static void main(String[] args) {
        String content = "反复发热1月余，体温最高39度，伴畏寒，咽痛，恶心呕吐，头痛，发热时有关节疼痛，无肿胀，无皮疹，无明显咳嗽咳痰，" +
                "无胸闷气急，无腹痛，无尿频尿急尿痛。外院2015.10.12两下肺少许感染性病变，血象、CRP轻度升高\n" +
                "直肠术后，现造瘘\n" +
                "目前发热原因不能以肺部感染解释，建议感染科就诊";
        try {
            RelationExtractionController re = new RelationExtractionController();
            Map<String, String> relations = re.cnnExtractRelation(content);

            List<String> symptoms = re.extractEntity(content);

            Map<String, String> result = new HashMap<>();
            for (String s : symptoms) {
                List<String> propsList = new ArrayList<>(3);
                for (Map.Entry<String, String> entry : relations.entrySet()) {
                    if (entry.getKey().equals(s)) {
                        propsList.add(entry.getValue());
                    }
                    if (entry.getValue().equals(s)) {
                        propsList.add(entry.getKey());
                    }
                }
                result.put(s, StringUtils.join(propsList, ","));
            }
            for (Map.Entry<String, String> e : result.entrySet()) {
                System.out.println(e.getKey() + " : " + e.getValue());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public List<String> extractEntity(String content) throws Exception {
        FeatureAnalyze sa = new FeatureAnalyze();
        //症状信息
        List<Map<String, Object>> featuresList =  sa.start(content, FeatureType.SYMPTOM);
        //体征信息
        featuresList.addAll(sa.start(content, FeatureType.VITAL));

        List<String> symptoms = new ArrayList<>(featuresList.size());
        for (Map<String, Object> m : featuresList) {
            if (!symptoms.contains(String.valueOf(m.get("feature_name")))) {
                symptoms.add(String.valueOf(m.get("feature_name")));
            }
        }
        return symptoms;
    }

    public Map<String, String> cnnExtractRelation(String content) {
        Map<String, String> map = new HashMap<>();
        map.put("反复", "发热");
        map.put("发热", "1月余");
        map.put("发热", "39度");
        map.put("体温", "39度");
        map.put("发热", "关节疼痛");
        map.put("肿胀", "无");
        map.put("皮疹", "无");
        map.put("咳嗽", "无");
        map.put("咳痰", "无");
        map.put("胸闷", "无");
        map.put("气急", "无");
        map.put("腹痛", "无");
        map.put("尿频", "无");
        map.put("尿急", "无");
        map.put("尿痛", "无");
        map.put("明显", "咳嗽");
        map.put("轻度", "升高");
        map.put("目前", "发热");
        return map;
    }
}
