package org.diagbot.nlp.participle;

import org.diagbot.nlp.participle.cfg.Configuration;
import org.diagbot.nlp.participle.cfg.DefaultConfig;
import org.diagbot.nlp.participle.word.Lexeme;
import org.diagbot.nlp.participle.word.LexemePath;
import org.diagbot.nlp.participle.word.Segment;
import org.diagbot.nlp.util.NlpUtil;
import org.diagbot.nlp.util.NlpCache;

import java.io.IOException;
import java.io.StringReader;

/**
 * Created by fyeman on 2018/1/31.
 */
public class ParticipleUtil {

    public static LexemePath<Lexeme> participle(String content, boolean isCombineUnit) throws IOException {
        LexemePath<Lexeme> lexemes = participle(content);

        LexemePath<Lexeme> results = new LexemePath<>();
        if (isCombineUnit) {
            Lexeme l = null;
            Lexeme last_l = null;
            for (int i = 0, len = lexemes.size(); i < len; i++) {
                l = lexemes.get(i);
                if (l.getProperty() != null && ("2".equals(l.getProperty()) || "9".equals(l.getProperty()))) {
                    if (i > 0) {
                        last_l = lexemes.get(i - 1);
                        if (NlpUtil.isNumberString(last_l)) {
                            last_l.setText(last_l.getText() + l.getText());
                            last_l.setProperty(l.getProperty());
                            last_l.setLength(last_l.getLength() +  l.getLength());

                            results.remove(results.size() - 1);
                            results.add(last_l);
                        } else {
                            results.add(l);
                        }
                    } else {
                        results.add(l);
                    }
                } else {
                    results.add(l);
                }
            }
        }
        return results;
    }

    public static LexemePath<Lexeme> participle(String content) throws IOException {
        if (NlpCache.segment_cache == null) {
            NlpCache.createSegmentCache();
        }
        return participle(content, NlpCache.segment_cache);
    }

    public static LexemePath<Lexeme> participle(String content, String path) throws IOException {
        Configuration configuration = new DefaultConfig();
        NlpCache.segment_cache = configuration.loadMainDict(path);
        return participle(content, NlpCache.segment_cache);
    }

    public static LexemePath<Lexeme> participle(String content, Segment segment) throws IOException {
        if (segment == null) return participle(content);
        ParticipleToken token = new ParticipleToken();
        NlpCache.segment_cache = segment;
        token.start(new StringReader(content), NlpCache.segment_cache);
        LexemePath<Lexeme> lexemePath = token.getLexemePath();
        token.end();

        return lexemePath;
    }

    public static String participleAndHighlight(String content) throws IOException {
        LexemePath<Lexeme> lexemePath = participle(content, true);
        String separator = "&nbsp;*&nbsp;";
        StringBuffer sb = new StringBuffer();
        for (Lexeme lexeme : lexemePath) {
            if (lexeme.getOffset() > -1) {
                if (lexeme.getProperty() != null && !"99".equals(lexeme.getProperty())) {
                    sb.append(separator);
                    sb.append("<font color='blue'>");
                    sb.append(lexeme.getText());
                    sb.append("</font>");
                } else {
                    sb.append(separator);
                    sb.append(lexeme.getText());
                }
            }
        }
        return sb.toString();
    }
}
